@@ -38,18 +38,9 @@ static inline bool feature_graphemes(void)
 #endif
 }
 
-static inline bool feature_fractional_scaling(void)
+static inline bool feature_xdg_toplevel_icon(void)
 {
-#if defined(HAVE_FRACTIONAL_SCALE)
-    return true;
-#else
-    return false;
-#endif
-}
-
-static inline bool feature_cursor_shape(void)
-{
-#if defined(HAVE_CURSOR_SHAPE)
+#if defined(HAVE_XDG_TOPLEVEL_ICON)
     return true;
 #else
     return false;
