@@ -547,20 +547,21 @@ osc_notify(struct terminal *term, char *string)
     if (title == NULL)
         return;
 
-    if (mbsntoc32(NULL, title, strlen(title), 0) == (char32_t)-1) {
+    if (mbsntoc32(NULL, title, strlen(title), 0) == (size_t)-1) {
         LOG_WARN("%s: notification title is not valid UTF-8, ignoring", title);
         return;
     }
 
-    if (msg != NULL && mbsntoc32(NULL, msg, strlen(msg), 0) == (char32_t)-1) {
+    if (msg != NULL && mbsntoc32(NULL, msg, strlen(msg), 0) == (size_t)-1) {
         LOG_WARN("%s: notification message is not valid UTF-8, ignoring", msg);
         return;
     }
 
     notify_notify(term, &(struct notification){
-        .title = (char *)title,
-        .body = (char *)msg,
+        .title = xstrdup(title),
+        .body = xstrdup(msg),
         .expire_time = -1,
+        .focus = true,
     });
 }
 
@@ -738,7 +739,7 @@ kitty_notification(struct terminal *term, char *string)
                 const char *terminator = term->vt.osc.bel ? "\a" : "\033\\";
 
                 char reply[128];
-                int n = xsnprintf(
+                size_t n = xsnprintf(
                     reply, sizeof(reply),
                     "\033]99;i=%s:p=?;p=%s:a=%s:o=%s:u=%s:c=1:w=1:s=system,silent,error,warn,warning,info,question%s",
                     reply_id, p_caps, a_caps, when_caps, u_caps, terminator);
@@ -1144,9 +1145,16 @@ osc_dispatch(struct terminal *term)
     char *string = (char *)&term->vt.osc.data[data_ofs];
 
     switch (param) {
-    case 0: term_set_window_title(term, string); break;  /* icon + title */
-    case 1: break;                                       /* icon */
-    case 2: term_set_window_title(term, string); break;  /* title */
+    case 0:  /* icon + title */
+        term_set_window_title(term, string);
+        break;
+
+    case 1:  /* icon */
+        break;
+
+    case 2:  /* title */
+        term_set_window_title(term, string);
+        break;
 
     case 4: {
         /* Set color<idx> */
