@@ -16,7 +16,7 @@
 #include "csi.h"
 #include "dcs.h"
 #include "debug.h"
-#include "grid.h"
+#include "emoji-variation-sequences.h"
 #include "osc.h"
 #include "sixel.h"
 #include "util.h"
@@ -242,12 +242,16 @@ action_execute(struct terminal *term, uint8_t c)
     case '\x0e':
         /* SO - shift out */
         term->charsets.selected = G1;
+        term->bits_affecting_ascii_printer.charset =
+            term->charsets.set[term->charsets.selected] != CHARSET_ASCII;
         term_update_ascii_printer(term);
         break;
 
     case '\x0f':
         /* SI - shift in */
         term->charsets.selected = G0;
+        term->bits_affecting_ascii_printer.charset =
+            term->charsets.set[term->charsets.selected] != CHARSET_ASCII;
         term_update_ascii_printer(term);
         break;
 
@@ -481,12 +485,16 @@ action_esc_dispatch(struct terminal *term, uint8_t final)
         case 'n':
             /* LS2 - Locking Shift 2 */
             term->charsets.selected = G2;
+            term->bits_affecting_ascii_printer.charset =
+                term->charsets.set[term->charsets.selected] != CHARSET_ASCII;
             term_update_ascii_printer(term);
             break;
 
         case 'o':
             /* LS3 - Locking Shift 3 */
             term->charsets.selected = G3;
+            term->bits_affecting_ascii_printer.charset =
+                term->charsets.set[term->charsets.selected] != CHARSET_ASCII;
             term_update_ascii_printer(term);
             break;
 
@@ -545,6 +553,8 @@ action_esc_dispatch(struct terminal *term, uint8_t final)
             size_t idx = term->vt.private - '(';
             xassert(idx <= G3);
             term->charsets.set[idx] = CHARSET_GRAPHIC;
+            term->bits_affecting_ascii_printer.charset =
+                term->charsets.set[term->charsets.selected] != CHARSET_ASCII;
             term_update_ascii_printer(term);
             break;
         }
@@ -553,6 +563,8 @@ action_esc_dispatch(struct terminal *term, uint8_t final)
             size_t idx = term->vt.private - '(';
             xassert(idx <= G3);
             term->charsets.set[idx] = CHARSET_ASCII;
+            term->bits_affecting_ascii_printer.charset =
+                term->charsets.set[term->charsets.selected] != CHARSET_ASCII;
             term_update_ascii_printer(term);
             break;
         }
@@ -655,6 +667,38 @@ chain_key(uint32_t old_key, uint32_t new_wc)
     return new_key;
 }
 
+#if defined(FOOT_GRAPHEME_CLUSTERING)
+static int
+emoji_vs_compare(const void *_key, const void *_entry)
+{
+    const struct emoji_vs *key = _key;
+    const struct emoji_vs *entry = _entry;
+
+    uint32_t cp = key->start;
+
+    if (cp < entry->start)
+        return -1;
+    else if (cp > entry->end)
+        return 1;
+    else
+        return 0;
+}
+
+UNITTEST
+{
+    /* Verify the emoji_vs list is sorted */
+    int64_t last_end = -1;
+
+    for (size_t i = 0; i < sizeof(emoji_vs) / sizeof(emoji_vs[0]); i++) {
+        const struct emoji_vs *vs = &emoji_vs[i];
+        xassert(vs->start <= vs->end);
+        xassert(vs->start > last_end);
+        xassert(vs->vs15 || vs->vs16);
+        last_end = vs->end;
+    }
+}
+#endif
+
 static void
 action_utf8_print(struct terminal *term, char32_t wc)
 {
@@ -852,21 +896,37 @@ action_utf8_print(struct terminal *term, char32_t wc)
                 break;
 
             case GRAPHEME_WIDTH_DOUBLE:
+                new_cc->width = min(grapheme_width + width, 2);
+
 #if defined(FOOT_GRAPHEME_CLUSTERING)
+                /* Handle VS-15 and VS-16 variation selectors */
                 if (unlikely(grapheme_clustering &&
-                             wc == 0xfe0f &&
+                             (wc == 0xfe0e || wc == 0xfe0f) &&
                              new_cc->count == 2))
                 {
-                    /* Only emojis should be affected by VS16 */
-                    const utf8proc_property_t *props =
-                        utf8proc_get_property(new_cc->chars[0]);
+                    const struct emoji_vs *vs =
+                        bsearch(
+                            &(struct emoji_vs){.start = new_cc->chars[0]},
+                            emoji_vs, sizeof(emoji_vs) / sizeof(emoji_vs[0]),
+                            sizeof(struct emoji_vs),
+                            &emoji_vs_compare);
 
-                    if (props->boundclass == UTF8PROC_BOUNDCLASS_EXTENDED_PICTOGRAPHIC)
-                        width = 2;
+                    if (vs != NULL) {
+                        xassert(new_cc->chars[0] >= vs->start &&
+                                new_cc->chars[0] <= vs->end);
+
+                        /* Force a grapheme width of 1 for VS-15, and 2 for VS-16 */
+                        if (wc == 0xfe0e) {
+                            if (vs->vs15)
+                                new_cc->width = 1;
+                        } else if (wc == 0xfe0f) {
+                            if (vs->vs16)
+                                new_cc->width = 2;
+                        }
+                    }
                 }
 #endif
 
-                new_cc->width = min(grapheme_width + width, 2);
                 break;
 
             case GRAPHEME_WIDTH_WCSWIDTH:
